﻿Imports Microsoft.VisualStudio.TestTools.UnitTesting

Imports EF6Model
Imports System.Data.Entity

Imports EF6Model.Models
Imports System.Data.Entity.Infrastructure

Namespace EF6WebAPI.Tests
  <TestClass>
  Public Class ContextEntityStateTests
	Public Sub New()
            Database.SetInitializer(New NullDatabaseInitializer(Of NinjaContext))

        End Sub
	<TestMethod>
	Public Sub AddSingleUntrackedEntityToContextResultsinAddedState()
            Dim ninja = New Ninja
            Using context = New NinjaContextDefaultBehavior
                context.Ninjas.Add(ninja)
                Assert.AreEqual(EntityState.Added, context.ChangeTracker.Entries.FirstOrDefault.State)
            End Using
        End Sub
	<TestMethod>
	Public Sub AddUntrackedGraphOfNewEntitiesResultsinAddedStateOnAll()
            Dim ninja = New Ninja
            ninja.EquipmentOwned.Add(New NinjaEquipment)
            Using context = New NinjaContextDefaultBehavior
                context.Ninjas.Add(ninja)
                Dim entries = context.ChangeTracker.Entries
                entries.ToList.ForEach(Sub(e) Debug.WriteLine($"{e.Entity.ToString}:{e.State}"))
                CollectionAssert.AreEqual({EntityState.Added, EntityState.Added}, entries.Select(Function(e) e.State).ToArray)
            End Using
        End Sub
	<TestMethod>
	Public Sub EFDoesNotComprehendsMixedStatesWhenAddingUntrackedGraph()
            Dim ninja = New Ninja
            ninja.Clan = New Clan With {.Id = 1}
            Using context = New NinjaContextDefaultBehavior
                context.Ninjas.Add(ninja)
                Dim entries = context.ChangeTracker.Entries
                entries.ToList.ForEach(Sub(e) Debug.WriteLine($"{e.Entity.ToString}:{e.State}"))
                Assert.IsFalse(entries.Any(Function(e) e.State <> EntityState.Added))
            End Using
        End Sub


	<TestMethod>
	Public Sub EFComprehendsMixedStatesWhenAddingUntrackedGraph()
	  Dim ninja = EF6Model.RichModels.Ninja.CreateIndependent("julie", True)
	  ninja.SpecifyClan(New EF6Model.RichModels.Clan With {.Id = 1, .ClanName = "Clan from database"})
            Using context = New NinjaContext
                context.Ninjas.Add(ninja)
                Dim entries = context.ChangeTracker.Entries
                OutputState(entries)
                context.FixState()
                OutputState(entries)
                Assert.IsTrue(entries.Any(Function(e) e.State = EntityState.Unchanged))

            End Using
        End Sub
	<TestMethod>
	Public Sub MixedStatesWithExistingParentAndVaryingChildrenisUnderstood()
	  Dim ninja = EF6Model.RichModels.Ninja.CreateIndependent("julie", True)
	  Dim pNinja = New PrivateObject(ninja)
	  pNinja.SetProperty("Id", 1)
	  Dim originalOwnerId = 99
	  Dim equip = EF6Model.RichModels.NinjaEquipment.Create(originalOwnerId, "arrow")
	  ninja.TransferEquipmentFromAnotherNinja(equip)
            Using context = New NinjaContext
                context.Ninjas.Attach(ninja)
                Dim entries = context.ChangeTracker.Entries
                OutputState(entries)
                context.FixState()
                OutputState(entries)
                Assert.IsTrue(entries.Any(Function(e) e.State = EntityState.Modified))

            End Using
        End Sub

	Private Shared Sub OutputState(ByVal entries As IEnumerable(Of DbEntityEntry))
            entries.ToList.ForEach(Sub(e) Debug.WriteLine($"Before:{e.Entity.ToString}:{e.State}"))
        End Sub
  End Class


End Namespace