﻿Imports Microsoft.VisualStudio.TestTools.UnitTesting

Imports EF6Model
Imports System.Data.Entity

Imports EF6Model.Models
Imports System.Text

Namespace EF6WebAPI.Tests
  <TestClass>
  Public Class VerifyDatabaseSavesTests
        Private _logBuilder As New StringBuilder
        Private _log As String


	Public Sub New()
            Database.SetInitializer(New DropCreateDatabaseAlways(Of NinjaContext))

        End Sub



	<TestMethod>
	Public Sub EFComprehendsMixedStatesWhenAddingUntrackedGraph()
	  Dim ninja = EF6Model.RichModels.Ninja.CreateIndependent("julie", True)
	  ninja.SpecifyClan(New EF6Model.RichModels.Clan With {.Id = 1, .ClanName = "Clan from database"})
            Using context = New NinjaContext
                context.Ninjas.Add(ninja)
                Dim entries = context.ChangeTracker.Entries
                entries.ToList.ForEach(Sub(e) Debug.WriteLine($"Before:{e.Entity.ToString}:{e.State}"))
                context.FixState()
                entries.ToList.ForEach(Sub(e) Debug.WriteLine($"After:{e.Entity.ToString}:{e.State}"))
                Assert.IsTrue(entries.Any(Function(e) e.State = EntityState.Unchanged))

            End Using
        End Sub
	<TestMethod>
	Public Sub MixedStatesWithExistingParentAndVaryingChildrenisUnderstood()

	  Dim ninja = EF6Model.RichModels.Ninja.CreateIndependent("julie", True)
	  Dim pNinja = New PrivateObject(ninja)
	  pNinja.SetProperty("Id", 1)
	  Dim originalOwnerId = 99
	  Dim equip = EF6Model.RichModels.NinjaEquipment.Create(originalOwnerId, "arrow")
	  ninja.TransferEquipmentFromAnotherNinja(equip)
            Using context = New NinjaContext
                context.Ninjas.Attach(ninja)
                Dim entries = context.ChangeTracker.Entries
                OutputState(entries)
                context.FixState()
                entries.ToList.ForEach(Sub(e) Debug.WriteLine($"After:{e.Entity.ToString}:{e.State}"))
                Assert.IsTrue(entries.Any(Function(e) e.State = EntityState.Modified))

            End Using
        End Sub

	Private Shared Sub OutputState(ByVal entries As IEnumerable(Of Entity.Infrastructure.DbEntityEntry))
            entries.ToList().ForEach(Sub(e) Debug.WriteLine($"Before:{e.Entity.ToString}:{e.State}"))
        End Sub

	<TestMethod>
	Public Sub EquipmentInNewNinjaGraphGetsNinjaIdViaSaveChanges()
	  Dim ninja = EF6Model.RichModels.Ninja.CreateIndependent("julie", True)
	  ninja.SpecifyClan(New EF6Model.RichModels.Clan With {.ClanName = "Vermont Ninjas", .State = EF6Model.Models.Enums.ObjectStates.Added})
	  ninja.AddEquipmentToNinja("arrow")
            Using context = New NinjaContext
                context.Ninjas.Add(ninja)
                Dim entries = context.ChangeTracker.Entries
                OutputState(entries)
                SetupLogging(context)
                context.SaveChanges()
                WriteLog()
            End Using
            Assert.AreEqual(ninja.Id, ninja.EquipmentOwned.First.NinjaId)
        End Sub

	<TestMethod>
	Public Sub CanMoveEquipmentFromOneNinjaToAnother()
	  SeedForTest()
	  Dim ninjas As List(Of EF6Model.RichModels.Ninja)
            Using context = New NinjaContext
                ninjas = context.Ninjas.AsNoTracking.Include(Function(n) n.EquipmentOwned).ToList
            End Using
            Dim rivalEquip = ninjas.Find(Function(n) n.Name = "rival").EquipmentOwned.FirstOrDefault
            Dim julie = ninjas.Find(Function(n) n.Name = "julie")
	  Dim originalEquipmentCount = julie.EquipmentOwned.Count
	  julie.TransferEquipmentFromAnotherNinja(rivalEquip)
            Using context = New NinjaContext
                context.Ninjas.Add(julie)
                Dim entries = context.ChangeTracker.Entries
                OutputState(entries)
                SetupLogging(context)
                context.SaveChanges()
                WriteLog()

            End Using
            Using context = New NinjaContext
                Dim juliefromDb = context.Ninjas.Include(Function(n) n.EquipmentOwned).Where(Function(n) n.Name = "julie").FirstOrDefault
                Assert.AreEqual(originalEquipmentCount + 1, juliefromDb.EquipmentOwned.Count)

            End Using

        End Sub


	Private Sub SeedForTest()
	  Dim ninja = EF6Model.RichModels.Ninja.CreateIndependent("julie", True)
	  ninja.SpecifyClan(New EF6Model.RichModels.Clan With {.ClanName = "Vermont Ninjas", .State = EF6Model.Models.Enums.ObjectStates.Added})
	  ninja.AddEquipmentToNinja("arrow")
	  Dim rival = EF6Model.RichModels.Ninja.CreateIndependent("rival", True)
	  rival.SpecifyClan(New EF6Model.RichModels.Clan With {.ClanName = "TraitorClan", .State = EF6Model.Models.Enums.ObjectStates.Added})
	  rival.AddEquipmentToNinja("cannon")
            Using context = New NinjaContext
                context.Ninjas.AddRange({ninja, rival})
                context.SaveChanges()
            End Using

        End Sub

	Private Sub WriteLog()
	  Debug.WriteLine(_log)

	End Sub

	Private Sub SetupLogging(ByVal context As NinjaContext)
	  _logBuilder.Clear()
	  context.Database.Log = AddressOf BuildLogString
	End Sub

	Private Sub BuildLogString(ByVal message As String)
	  _logBuilder.Append(message)
            _log = _logBuilder.ToString
        End Sub
  End Class
End Namespace
